@@ -4,12 +4,18 @@
 
 
 import os
+import pathlib
+import platform
 import sys
 from distutils import dist
 from distutils.ccompiler import get_default_compiler
 from distutils.command.config import config
 
-from _cffi_src.utils import build_ffi_for_binding, compiler_type
+# Add the src directory to the path so we can import _cffi_src.utils
+src_dir = str(pathlib.Path(__file__).parent.parent)
+sys.path.insert(0, src_dir)
+
+from _cffi_src.utils import build_ffi_for_binding, compiler_type  # noqa: E402
 
 
 def _get_openssl_libraries(platform):
@@ -70,7 +76,7 @@ def _extra_compile_args(platform):
 
 
 ffi = build_ffi_for_binding(
-    module_name="cryptography.hazmat.bindings._openssl",
+    module_name="_openssl",
     module_prefix="_cffi_src.openssl.",
     modules=[
         # This goes first so we can define some cryptography-wide symbols.
@@ -110,3 +116,17 @@ ffi = build_ffi_for_binding(
     libraries=_get_openssl_libraries(sys.platform),
     extra_compile_args=_extra_compile_args(sys.platform),
 )
+
+if __name__ == "__main__":
+    out_dir = os.getenv("OUT_DIR")
+    module_name, source, source_extension, kwds = ffi._assigned_source
+    c_file = os.path.join(out_dir, module_name + source_extension)
+    if platform.python_implementation() == "PyPy":
+        # Necessary because CFFI will ignore this if there's no declarations.
+        ffi.embedding_api(
+            """
+            extern "Python" void Cryptography_unused(void);
+        """
+        )
+        ffi.embedding_init_code("")
+    ffi.emit_c_code(c_file)
