@@ -17,8 +17,9 @@
 #include <poll.h>
 #endif
 
-#ifndef OPENSSL_NO_ENGINE
-/* OpenSSL has ENGINE support so build the engine. */
+#if CRYPTOGRAPHY_NEEDS_OSRANDOM_ENGINE
+/* OpenSSL has ENGINE support and is older than 1.1.1d (the first version that
+ * properly implements fork safety in its RNG) so build the engine. */
 static const char *Cryptography_osrandom_engine_id = "osrandom";
 
 /****************************************************************************
@@ -245,22 +246,7 @@ static void dev_urandom_close(void) {
 #if CRYPTOGRAPHY_OSRANDOM_ENGINE == CRYPTOGRAPHY_OSRANDOM_ENGINE_GETENTROPY
 static const char *Cryptography_osrandom_engine_name = "osrandom_engine getentropy()";
 
-static int getentropy_works = CRYPTOGRAPHY_OSRANDOM_GETENTROPY_NOT_INIT;
-
 static int osrandom_init(ENGINE *e) {
-#if !defined(__APPLE__)
-    getentropy_works = CRYPTOGRAPHY_OSRANDOM_GETENTROPY_WORKS;
-#else
-    if (__builtin_available(macOS 10.12, *)) {
-        getentropy_works = CRYPTOGRAPHY_OSRANDOM_GETENTROPY_WORKS;
-    } else {
-        getentropy_works = CRYPTOGRAPHY_OSRANDOM_GETENTROPY_FALLBACK;
-        int fd = dev_urandom_fd();
-        if (fd < 0) {
-            return 0;
-        }
-    }
-#endif
     return 1;
 }
 
@@ -268,34 +254,22 @@ static int osrandom_rand_bytes(unsigned char *buffer, int size) {
     int len;
     int res;
 
-    switch(getentropy_works) {
-#if defined(__APPLE__)
-    case CRYPTOGRAPHY_OSRANDOM_GETENTROPY_FALLBACK:
-        return dev_urandom_read(buffer, size);
-#endif
-    case CRYPTOGRAPHY_OSRANDOM_GETENTROPY_WORKS:
-        while (size > 0) {
-            /* OpenBSD and macOS restrict maximum buffer size to 256. */
-            len = size > 256 ? 256 : size;
-/* on mac, availability is already checked using `__builtin_available` above */
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wunguarded-availability"
-            res = getentropy(buffer, (size_t)len);
-#pragma clang diagnostic pop
-            if (res < 0) {
-                ERR_Cryptography_OSRandom_error(
-                    CRYPTOGRAPHY_OSRANDOM_F_RAND_BYTES,
-                    CRYPTOGRAPHY_OSRANDOM_R_GETENTROPY_FAILED,
-                    __FILE__, __LINE__
-                );
-                return 0;
-            }
-            buffer += len;
-            size -= len;
+    while (size > 0) {
+        /* OpenBSD and macOS restrict maximum buffer size to 256. */
+        len = size > 256 ? 256 : size;
+        res = getentropy(buffer, (size_t)len);
+        if (res < 0) {
+            ERR_Cryptography_OSRandom_error(
+                CRYPTOGRAPHY_OSRANDOM_F_RAND_BYTES,
+                CRYPTOGRAPHY_OSRANDOM_R_GETENTROPY_FAILED,
+                __FILE__, __LINE__
+            );
+            return 0;
         }
-        return 1;
+        buffer += len;
+        size -= len;
     }
-    __builtin_unreachable();
+    return 1;
 }
 
 static int osrandom_finish(ENGINE *e) {
@@ -307,13 +281,7 @@ static int osrandom_rand_status(void) {
 }
 
 static const char *osurandom_get_implementation(void) {
-    switch(getentropy_works) {
-    case CRYPTOGRAPHY_OSRANDOM_GETENTROPY_FALLBACK:
-        return "/dev/urandom";
-    case CRYPTOGRAPHY_OSRANDOM_GETENTROPY_WORKS:
-        return "getentropy";
-    }
-    __builtin_unreachable();
+    return "getentropy";
 }
 #endif /* CRYPTOGRAPHY_OSRANDOM_ENGINE_GETENTROPY */
 
@@ -528,7 +496,7 @@ static int osrandom_ctrl(ENGINE *e, int cmd, long i, void *p, void (*f) (void))
             ENGINEerr(ENGINE_F_ENGINE_CTRL, ENGINE_R_INVALID_ARGUMENT);
             return 0;
         }
-        strncpy((char *)p, name, len);
+        strcpy((char *)p, name);
         return (int)len;
     default:
         ENGINEerr(ENGINE_F_ENGINE_CTRL, ENGINE_R_CTRL_COMMAND_NOT_IMPLEMENTED);
@@ -650,7 +618,7 @@ int Cryptography_add_osrandom_engine(void) {
  * to compile the osrandom engine, but we do need some
  * placeholders */
 static const char *Cryptography_osrandom_engine_id = "no-engine-support";
-static const char *Cryptography_osrandom_engine_name = "osrandom_engine disabled due to no engine support";
+static const char *Cryptography_osrandom_engine_name = "osrandom_engine disabled";
 
 int Cryptography_add_osrandom_engine(void) {
     return 0;
