@@ -2,104 +2,62 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
 
-from cryptography import utils
+import typing
+
 from cryptography.exceptions import InvalidSignature
-from cryptography.hazmat.backends.openssl.utils import _truncate_digest
-from cryptography.hazmat.primitives import hashes, serialization
-from cryptography.hazmat.primitives.asymmetric import (
-    AsymmetricSignatureContext, AsymmetricVerificationContext, dsa
+from cryptography.hazmat.backends.openssl.utils import (
+    _calculate_digest_and_algorithm,
 )
+from cryptography.hazmat.primitives import hashes, serialization
+from cryptography.hazmat.primitives.asymmetric import dsa
+from cryptography.hazmat.primitives.asymmetric import utils as asym_utils
+
+if typing.TYPE_CHECKING:
+    from cryptography.hazmat.backends.openssl.backend import Backend
 
 
-def _truncate_digest_for_dsa(dsa_cdata, digest, backend):
-    """
-    This function truncates digests that are longer than a given DS
-    key's length so they can be signed. OpenSSL does this for us in
-    1.0.0c+, leaving us with three releases (1.0.0, 1.0.0a, and 1.0.0b) where
-    this is a problem.
-    """
+def _dsa_sig_sign(
+    backend: "Backend", private_key: "_DSAPrivateKey", data: bytes
+) -> bytes:
+    sig_buf_len = backend._lib.DSA_size(private_key._dsa_cdata)
+    sig_buf = backend._ffi.new("unsigned char[]", sig_buf_len)
+    buflen = backend._ffi.new("unsigned int *")
 
-    q = backend._ffi.new("BIGNUM **")
-    backend._lib.DSA_get0_pqg(
-        dsa_cdata, backend._ffi.NULL, q, backend._ffi.NULL
+    # The first parameter passed to DSA_sign is unused by OpenSSL but
+    # must be an integer.
+    res = backend._lib.DSA_sign(
+        0, data, len(data), sig_buf, buflen, private_key._dsa_cdata
     )
-    backend.openssl_assert(q[0] != backend._ffi.NULL)
+    backend.openssl_assert(res == 1)
+    backend.openssl_assert(buflen[0])
 
-    order_bits = backend._lib.BN_num_bits(q[0])
-    return _truncate_digest(digest, order_bits)
+    return backend._ffi.buffer(sig_buf)[: buflen[0]]
 
 
-@utils.register_interface(AsymmetricVerificationContext)
-class _DSAVerificationContext(object):
-    def __init__(self, backend, public_key, signature, algorithm):
-        self._backend = backend
-        self._public_key = public_key
-        self._signature = signature
-        self._algorithm = algorithm
+def _dsa_sig_verify(
+    backend: "Backend",
+    public_key: "_DSAPublicKey",
+    signature: bytes,
+    data: bytes,
+) -> None:
+    # The first parameter passed to DSA_verify is unused by OpenSSL but
+    # must be an integer.
+    res = backend._lib.DSA_verify(
+        0, data, len(data), signature, len(signature), public_key._dsa_cdata
+    )
 
-        self._hash_ctx = hashes.Hash(self._algorithm, self._backend)
-
-    def update(self, data):
-        self._hash_ctx.update(data)
-
-    def verify(self):
-        data_to_verify = self._hash_ctx.finalize()
-
-        data_to_verify = _truncate_digest_for_dsa(
-            self._public_key._dsa_cdata, data_to_verify, self._backend
-        )
-
-        # The first parameter passed to DSA_verify is unused by OpenSSL but
-        # must be an integer.
-        res = self._backend._lib.DSA_verify(
-            0, data_to_verify, len(data_to_verify), self._signature,
-            len(self._signature), self._public_key._dsa_cdata)
-
-        if res != 1:
-            self._backend._consume_errors()
-            raise InvalidSignature
+    if res != 1:
+        backend._consume_errors()
+        raise InvalidSignature
 
 
-@utils.register_interface(AsymmetricSignatureContext)
-class _DSASignatureContext(object):
-    def __init__(self, backend, private_key, algorithm):
-        self._backend = backend
-        self._private_key = private_key
-        self._algorithm = algorithm
-        self._hash_ctx = hashes.Hash(self._algorithm, self._backend)
-
-    def update(self, data):
-        self._hash_ctx.update(data)
-
-    def finalize(self):
-        data_to_sign = self._hash_ctx.finalize()
-        data_to_sign = _truncate_digest_for_dsa(
-            self._private_key._dsa_cdata, data_to_sign, self._backend
-        )
-        sig_buf_len = self._backend._lib.DSA_size(self._private_key._dsa_cdata)
-        sig_buf = self._backend._ffi.new("unsigned char[]", sig_buf_len)
-        buflen = self._backend._ffi.new("unsigned int *")
-
-        # The first parameter passed to DSA_sign is unused by OpenSSL but
-        # must be an integer.
-        res = self._backend._lib.DSA_sign(
-            0, data_to_sign, len(data_to_sign), sig_buf,
-            buflen, self._private_key._dsa_cdata)
-        self._backend.openssl_assert(res == 1)
-        self._backend.openssl_assert(buflen[0])
-
-        return self._backend._ffi.buffer(sig_buf)[:buflen[0]]
-
-
-@utils.register_interface(dsa.DSAParametersWithNumbers)
-class _DSAParameters(object):
-    def __init__(self, backend, dsa_cdata):
+class _DSAParameters(dsa.DSAParameters):
+    def __init__(self, backend: "Backend", dsa_cdata):
         self._backend = backend
         self._dsa_cdata = dsa_cdata
 
-    def parameter_numbers(self):
+    def parameter_numbers(self) -> dsa.DSAParameterNumbers:
         p = self._backend._ffi.new("BIGNUM **")
         q = self._backend._ffi.new("BIGNUM **")
         g = self._backend._ffi.new("BIGNUM **")
@@ -110,16 +68,17 @@ class _DSAParameters(object):
         return dsa.DSAParameterNumbers(
             p=self._backend._bn_to_int(p[0]),
             q=self._backend._bn_to_int(q[0]),
-            g=self._backend._bn_to_int(g[0])
+            g=self._backend._bn_to_int(g[0]),
         )
 
-    def generate_private_key(self):
+    def generate_private_key(self) -> dsa.DSAPrivateKey:
         return self._backend.generate_dsa_private_key(self)
 
 
-@utils.register_interface(dsa.DSAPrivateKeyWithSerialization)
-class _DSAPrivateKey(object):
-    def __init__(self, backend, dsa_cdata, evp_pkey):
+class _DSAPrivateKey(dsa.DSAPrivateKey):
+    _key_size: int
+
+    def __init__(self, backend: "Backend", dsa_cdata, evp_pkey):
         self._backend = backend
         self._dsa_cdata = dsa_cdata
         self._evp_pkey = evp_pkey
@@ -131,12 +90,11 @@ class _DSAPrivateKey(object):
         self._backend.openssl_assert(p[0] != backend._ffi.NULL)
         self._key_size = self._backend._lib.BN_num_bits(p[0])
 
-    key_size = utils.read_only_property("_key_size")
+    @property
+    def key_size(self) -> int:
+        return self._key_size
 
-    def signer(self, signature_algorithm):
-        return _DSASignatureContext(self._backend, self, signature_algorithm)
-
-    def private_numbers(self):
+    def private_numbers(self) -> dsa.DSAPrivateNumbers:
         p = self._backend._ffi.new("BIGNUM **")
         q = self._backend._ffi.new("BIGNUM **")
         g = self._backend._ffi.new("BIGNUM **")
@@ -154,14 +112,14 @@ class _DSAPrivateKey(object):
                 parameter_numbers=dsa.DSAParameterNumbers(
                     p=self._backend._bn_to_int(p[0]),
                     q=self._backend._bn_to_int(q[0]),
-                    g=self._backend._bn_to_int(g[0])
+                    g=self._backend._bn_to_int(g[0]),
                 ),
-                y=self._backend._bn_to_int(pub_key[0])
+                y=self._backend._bn_to_int(pub_key[0]),
             ),
-            x=self._backend._bn_to_int(priv_key[0])
+            x=self._backend._bn_to_int(priv_key[0]),
         )
 
-    def public_key(self):
+    def public_key(self) -> dsa.DSAPublicKey:
         dsa_cdata = self._backend._lib.DSAparams_dup(self._dsa_cdata)
         self._backend.openssl_assert(dsa_cdata != self._backend._ffi.NULL)
         dsa_cdata = self._backend._ffi.gc(
@@ -180,7 +138,7 @@ class _DSAPrivateKey(object):
         evp_pkey = self._backend._dsa_cdata_to_evp_pkey(dsa_cdata)
         return _DSAPublicKey(self._backend, dsa_cdata, evp_pkey)
 
-    def parameters(self):
+    def parameters(self) -> dsa.DSAParameters:
         dsa_cdata = self._backend._lib.DSAparams_dup(self._dsa_cdata)
         self._backend.openssl_assert(dsa_cdata != self._backend._ffi.NULL)
         dsa_cdata = self._backend._ffi.gc(
@@ -188,24 +146,34 @@ class _DSAPrivateKey(object):
         )
         return _DSAParameters(self._backend, dsa_cdata)
 
-    def private_bytes(self, encoding, format, encryption_algorithm):
+    def private_bytes(
+        self,
+        encoding: serialization.Encoding,
+        format: serialization.PrivateFormat,
+        encryption_algorithm: serialization.KeySerializationEncryption,
+    ) -> bytes:
         return self._backend._private_key_bytes(
             encoding,
             format,
             encryption_algorithm,
+            self,
             self._evp_pkey,
-            self._dsa_cdata
+            self._dsa_cdata,
         )
 
-    def sign(self, data, algorithm):
-        signer = self.signer(algorithm)
-        signer.update(data)
-        return signer.finalize()
+    def sign(
+        self,
+        data: bytes,
+        algorithm: typing.Union[asym_utils.Prehashed, hashes.HashAlgorithm],
+    ) -> bytes:
+        data, _ = _calculate_digest_and_algorithm(data, algorithm)
+        return _dsa_sig_sign(self._backend, self, data)
 
 
-@utils.register_interface(dsa.DSAPublicKeyWithSerialization)
-class _DSAPublicKey(object):
-    def __init__(self, backend, dsa_cdata, evp_pkey):
+class _DSAPublicKey(dsa.DSAPublicKey):
+    _key_size: int
+
+    def __init__(self, backend: "Backend", dsa_cdata, evp_pkey):
         self._backend = backend
         self._dsa_cdata = dsa_cdata
         self._evp_pkey = evp_pkey
@@ -216,17 +184,11 @@ class _DSAPublicKey(object):
         self._backend.openssl_assert(p[0] != backend._ffi.NULL)
         self._key_size = self._backend._lib.BN_num_bits(p[0])
 
-    key_size = utils.read_only_property("_key_size")
+    @property
+    def key_size(self) -> int:
+        return self._key_size
 
-    def verifier(self, signature, signature_algorithm):
-        if not isinstance(signature, bytes):
-            raise TypeError("signature must be bytes.")
-
-        return _DSAVerificationContext(
-            self._backend, self, signature, signature_algorithm
-        )
-
-    def public_numbers(self):
+    def public_numbers(self) -> dsa.DSAPublicNumbers:
         p = self._backend._ffi.new("BIGNUM **")
         q = self._backend._ffi.new("BIGNUM **")
         g = self._backend._ffi.new("BIGNUM **")
@@ -243,33 +205,32 @@ class _DSAPublicKey(object):
             parameter_numbers=dsa.DSAParameterNumbers(
                 p=self._backend._bn_to_int(p[0]),
                 q=self._backend._bn_to_int(q[0]),
-                g=self._backend._bn_to_int(g[0])
+                g=self._backend._bn_to_int(g[0]),
             ),
-            y=self._backend._bn_to_int(pub_key[0])
+            y=self._backend._bn_to_int(pub_key[0]),
         )
 
-    def parameters(self):
+    def parameters(self) -> dsa.DSAParameters:
         dsa_cdata = self._backend._lib.DSAparams_dup(self._dsa_cdata)
         dsa_cdata = self._backend._ffi.gc(
             dsa_cdata, self._backend._lib.DSA_free
         )
         return _DSAParameters(self._backend, dsa_cdata)
 
-    def public_bytes(self, encoding, format):
-        if format is serialization.PublicFormat.PKCS1:
-            raise ValueError(
-                "DSA public keys do not support PKCS1 serialization"
-            )
-
+    def public_bytes(
+        self,
+        encoding: serialization.Encoding,
+        format: serialization.PublicFormat,
+    ) -> bytes:
         return self._backend._public_key_bytes(
-            encoding,
-            format,
-            self,
-            self._evp_pkey,
-            None
+            encoding, format, self, self._evp_pkey, None
         )
 
-    def verify(self, signature, data, algorithm):
-        verifier = self.verifier(signature, algorithm)
-        verifier.update(data)
-        verifier.verify()
+    def verify(
+        self,
+        signature: bytes,
+        data: bytes,
+        algorithm: typing.Union[asym_utils.Prehashed, hashes.HashAlgorithm],
+    ) -> None:
+        data, _ = _calculate_digest_and_algorithm(data, algorithm)
+        return _dsa_sig_verify(self._backend, self, signature, data)
