@@ -2,7 +2,9 @@
 // 2.0, and the BSD License. See the LICENSE file in the root of this repository
 // for complete details.
 
-use crate::asn1::{encode_der_data, PyAsn1Result};
+use crate::asn1::encode_der_data;
+use crate::buf::CffiBuf;
+use crate::error::CryptographyResult;
 use crate::x509;
 
 use chrono::Timelike;
@@ -32,7 +34,6 @@ static EMPTY_STRING_TLV: Lazy<asn1::Tlv<'static>> =
 
 static OIDS_TO_MIC_NAME: Lazy<HashMap<&asn1::ObjectIdentifier, &str>> = Lazy::new(|| {
     let mut h = HashMap::new();
-    h.insert(&x509::oid::SHA1_OID, "sha1");
     h.insert(&x509::oid::SHA224_OID, "sha-224");
     h.insert(&x509::oid::SHA256_OID, "sha-256");
     h.insert(&x509::oid::SHA384_OID, "sha-384");
@@ -88,7 +89,7 @@ fn serialize_certificates<'p>(
     py: pyo3::Python<'p>,
     py_certs: Vec<pyo3::PyRef<'p, x509::Certificate>>,
     encoding: &'p pyo3::PyAny,
-) -> PyAsn1Result<&'p pyo3::types::PyBytes> {
+) -> CryptographyResult<&'p pyo3::types::PyBytes> {
     if py_certs.is_empty() {
         return Err(pyo3::exceptions::PyTypeError::new_err(
             "certs must be a list of certs with length >= 1",
@@ -130,18 +131,21 @@ fn sign_and_serialize<'p>(
     builder: &'p pyo3::PyAny,
     encoding: &'p pyo3::PyAny,
     options: &'p pyo3::types::PyList,
-) -> PyAsn1Result<&'p pyo3::types::PyBytes> {
+) -> CryptographyResult<&'p pyo3::types::PyBytes> {
     let pkcs7_options = py
         .import("cryptography.hazmat.primitives.serialization.pkcs7")?
         .getattr(crate::intern!(py, "PKCS7Options"))?;
 
-    let raw_data = builder.getattr(crate::intern!(py, "_data"))?.extract()?;
+    let raw_data: CffiBuf<'p> = builder.getattr(crate::intern!(py, "_data"))?.extract()?;
     let text_mode = options.contains(pkcs7_options.getattr(crate::intern!(py, "Text"))?)?;
     let (data_with_header, data_without_header) =
         if options.contains(pkcs7_options.getattr(crate::intern!(py, "Binary"))?)? {
-            (Cow::Borrowed(raw_data), Cow::Borrowed(raw_data))
+            (
+                Cow::Borrowed(raw_data.as_bytes()),
+                Cow::Borrowed(raw_data.as_bytes()),
+            )
         } else {
-            smime_canonicalize(raw_data, text_mode)
+            smime_canonicalize(raw_data.as_bytes(), text_mode)
         };
 
     let content_type_bytes = asn1::write_single(&PKCS7_DATA_OID)?;
