@@ -2,40 +2,44 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
 
+import email.parser
 import os
+import typing
 
 import pytest
 
 from cryptography import x509
 from cryptography.exceptions import _Reasons
 from cryptography.hazmat.primitives import hashes, serialization
-from cryptography.hazmat.primitives.asymmetric import ed25519
+from cryptography.hazmat.primitives.asymmetric import ed25519, rsa
 from cryptography.hazmat.primitives.serialization import pkcs7
 
-from .utils import load_vectors_from_file
-from ...utils import raises_unsupported_algorithm
+from ...utils import load_vectors_from_file, raises_unsupported_algorithm
 
 
-class TestPKCS7Loading(object):
-    def test_load_invalid_der_pkcs7(self):
+@pytest.mark.supported(
+    only_if=lambda backend: backend.pkcs7_supported(),
+    skip_message="Requires OpenSSL with PKCS7 support",
+)
+class TestPKCS7Loading:
+    def test_load_invalid_der_pkcs7(self, backend):
         with pytest.raises(ValueError):
             pkcs7.load_der_pkcs7_certificates(b"nonsense")
 
-    def test_load_invalid_pem_pkcs7(self):
+    def test_load_invalid_pem_pkcs7(self, backend):
         with pytest.raises(ValueError):
             pkcs7.load_pem_pkcs7_certificates(b"nonsense")
 
-    def test_not_bytes_der(self):
+    def test_not_bytes_der(self, backend):
         with pytest.raises(TypeError):
-            pkcs7.load_der_pkcs7_certificates(38)
+            pkcs7.load_der_pkcs7_certificates(38)  # type: ignore[arg-type]
 
-    def test_not_bytes_pem(self):
+    def test_not_bytes_pem(self, backend):
         with pytest.raises(TypeError):
-            pkcs7.load_pem_pkcs7_certificates(38)
+            pkcs7.load_pem_pkcs7_certificates(38)  # type: ignore[arg-type]
 
-    def test_load_pkcs7_pem(self):
+    def test_load_pkcs7_pem(self, backend):
         certs = load_vectors_from_file(
             os.path.join("pkcs7", "isrg.pem"),
             lambda pemfile: pkcs7.load_pem_pkcs7_certificates(pemfile.read()),
@@ -44,13 +48,18 @@ class TestPKCS7Loading(object):
         assert len(certs) == 1
         assert certs[0].subject.get_attributes_for_oid(
             x509.oid.NameOID.COMMON_NAME
-        ) == [
-            x509.NameAttribute(x509.oid.NameOID.COMMON_NAME, u"ISRG Root X1")
-        ]
+        ) == [x509.NameAttribute(x509.oid.NameOID.COMMON_NAME, "ISRG Root X1")]
 
-    def test_load_pkcs7_der(self):
-        certs = load_vectors_from_file(
+    @pytest.mark.parametrize(
+        "filepath",
+        [
+            os.path.join("pkcs7", "amazon-roots.der"),
             os.path.join("pkcs7", "amazon-roots.p7b"),
+        ],
+    )
+    def test_load_pkcs7_der(self, filepath, backend):
+        certs = load_vectors_from_file(
+            filepath,
             lambda derfile: pkcs7.load_der_pkcs7_certificates(derfile.read()),
             mode="rb",
         )
@@ -59,18 +68,18 @@ class TestPKCS7Loading(object):
             x509.oid.NameOID.COMMON_NAME
         ) == [
             x509.NameAttribute(
-                x509.oid.NameOID.COMMON_NAME, u"Amazon Root CA 3"
+                x509.oid.NameOID.COMMON_NAME, "Amazon Root CA 3"
             )
         ]
         assert certs[1].subject.get_attributes_for_oid(
             x509.oid.NameOID.COMMON_NAME
         ) == [
             x509.NameAttribute(
-                x509.oid.NameOID.COMMON_NAME, u"Amazon Root CA 2"
+                x509.oid.NameOID.COMMON_NAME, "Amazon Root CA 2"
             )
         ]
 
-    def test_load_pkcs7_unsupported_type(self):
+    def test_load_pkcs7_unsupported_type(self, backend):
         with raises_unsupported_algorithm(_Reasons.UNSUPPORTED_SERIALIZATION):
             load_vectors_from_file(
                 os.path.join("pkcs7", "enveloped.pem"),
@@ -106,8 +115,12 @@ def _pkcs7_verify(encoding, sig, msg, certs, options, backend):
     store = backend._lib.X509_STORE_new()
     backend.openssl_assert(store != backend._ffi.NULL)
     store = backend._ffi.gc(store, backend._lib.X509_STORE_free)
+    # This list is to keep the x509 values alive until end of function
+    ossl_certs = []
     for cert in certs:
-        res = backend._lib.X509_STORE_add_cert(store, cert._x509)
+        ossl_cert = backend._cert2ossl(cert)
+        ossl_certs.append(ossl_cert)
+        res = backend._lib.X509_STORE_add_cert(store, ossl_cert)
         backend.openssl_assert(res == 1)
     if msg is None:
         res = backend._lib.PKCS7_verify(
@@ -120,17 +133,24 @@ def _pkcs7_verify(encoding, sig, msg, certs, options, backend):
         )
     else:
         msg_bio = backend._bytes_to_bio(msg)
+        # libressl 3.7.0 has a bug when NULL is passed as an `out_bio`. Work
+        # around it for now.
+        out_bio = backend._create_mem_bio_gc()
         res = backend._lib.PKCS7_verify(
-            p7, backend._ffi.NULL, store, msg_bio.bio, backend._ffi.NULL, flags
+            p7, backend._ffi.NULL, store, msg_bio.bio, out_bio, flags
         )
     backend.openssl_assert(res == 1)
+    # OpenSSL 3.0 leaves a random bio error on the stack:
+    # https://github.com/openssl/openssl/issues/16681
+    if backend._lib.CRYPTOGRAPHY_OPENSSL_300_OR_GREATER:
+        backend._consume_errors()
 
 
 def _load_cert_key():
     key = load_vectors_from_file(
         os.path.join("x509", "custom", "ca", "ca_key.pem"),
         lambda pemfile: serialization.load_pem_private_key(
-            pemfile.read(), None
+            pemfile.read(), None, unsafe_skip_rsa_key_validation=True
         ),
         mode="rb",
     )
@@ -142,23 +162,27 @@ def _load_cert_key():
     return cert, key
 
 
-class TestPKCS7Builder(object):
-    def test_invalid_data(self):
+@pytest.mark.supported(
+    only_if=lambda backend: backend.pkcs7_supported(),
+    skip_message="Requires OpenSSL with PKCS7 support",
+)
+class TestPKCS7Builder:
+    def test_invalid_data(self, backend):
         builder = pkcs7.PKCS7SignatureBuilder()
         with pytest.raises(TypeError):
-            builder.set_data(u"not bytes")
+            builder.set_data("not bytes")  # type: ignore[arg-type]
 
-    def test_set_data_twice(self):
+    def test_set_data_twice(self, backend):
         builder = pkcs7.PKCS7SignatureBuilder().set_data(b"test")
         with pytest.raises(ValueError):
             builder.set_data(b"test")
 
-    def test_sign_no_signer(self):
+    def test_sign_no_signer(self, backend):
         builder = pkcs7.PKCS7SignatureBuilder().set_data(b"test")
         with pytest.raises(ValueError):
             builder.sign(serialization.Encoding.SMIME, [])
 
-    def test_sign_no_data(self):
+    def test_sign_no_data(self, backend):
         cert, key = _load_cert_key()
         builder = pkcs7.PKCS7SignatureBuilder().add_signer(
             cert, key, hashes.SHA256()
@@ -166,18 +190,18 @@ class TestPKCS7Builder(object):
         with pytest.raises(ValueError):
             builder.sign(serialization.Encoding.SMIME, [])
 
-    def test_unsupported_hash_alg(self):
+    def test_unsupported_hash_alg(self, backend):
         cert, key = _load_cert_key()
         with pytest.raises(TypeError):
             pkcs7.PKCS7SignatureBuilder().add_signer(
-                cert, key, hashes.SHA512_256()
+                cert, key, hashes.SHA512_256()  # type: ignore[arg-type]
             )
 
-    def test_not_a_cert(self):
+    def test_not_a_cert(self, backend):
         cert, key = _load_cert_key()
         with pytest.raises(TypeError):
             pkcs7.PKCS7SignatureBuilder().add_signer(
-                b"notacert", key, hashes.SHA256()
+                b"notacert", key, hashes.SHA256()  # type: ignore[arg-type]
             )
 
     @pytest.mark.supported(
@@ -189,10 +213,10 @@ class TestPKCS7Builder(object):
         key = ed25519.Ed25519PrivateKey.generate()
         with pytest.raises(TypeError):
             pkcs7.PKCS7SignatureBuilder().add_signer(
-                cert, key, hashes.SHA256()
+                cert, key, hashes.SHA256()  # type: ignore[arg-type]
             )
 
-    def test_sign_invalid_options(self):
+    def test_sign_invalid_options(self, backend):
         cert, key = _load_cert_key()
         builder = (
             pkcs7.PKCS7SignatureBuilder()
@@ -200,9 +224,12 @@ class TestPKCS7Builder(object):
             .add_signer(cert, key, hashes.SHA256())
         )
         with pytest.raises(ValueError):
-            builder.sign(serialization.Encoding.SMIME, [b"invalid"])
+            builder.sign(
+                serialization.Encoding.SMIME,
+                [b"invalid"],  # type: ignore[list-item]
+            )
 
-    def test_sign_invalid_encoding(self):
+    def test_sign_invalid_encoding(self, backend):
         cert, key = _load_cert_key()
         builder = (
             pkcs7.PKCS7SignatureBuilder()
@@ -212,7 +239,7 @@ class TestPKCS7Builder(object):
         with pytest.raises(ValueError):
             builder.sign(serialization.Encoding.Raw, [])
 
-    def test_sign_invalid_options_text_no_detached(self):
+    def test_sign_invalid_options_text_no_detached(self, backend):
         cert, key = _load_cert_key()
         builder = (
             pkcs7.PKCS7SignatureBuilder()
@@ -223,7 +250,7 @@ class TestPKCS7Builder(object):
         with pytest.raises(ValueError):
             builder.sign(serialization.Encoding.SMIME, options)
 
-    def test_sign_invalid_options_text_der_encoding(self):
+    def test_sign_invalid_options_text_der_encoding(self, backend):
         cert, key = _load_cert_key()
         builder = (
             pkcs7.PKCS7SignatureBuilder()
@@ -237,7 +264,7 @@ class TestPKCS7Builder(object):
         with pytest.raises(ValueError):
             builder.sign(serialization.Encoding.DER, options)
 
-    def test_sign_invalid_options_no_attrs_and_no_caps(self):
+    def test_sign_invalid_options_no_attrs_and_no_caps(self, backend):
         cert, key = _load_cert_key()
         builder = (
             pkcs7.PKCS7SignatureBuilder()
@@ -263,6 +290,7 @@ class TestPKCS7Builder(object):
 
         sig = builder.sign(serialization.Encoding.SMIME, options)
         sig_binary = builder.sign(serialization.Encoding.DER, options)
+        assert b"text/plain" not in sig
         # We don't have a generic ASN.1 parser available to us so we instead
         # will assert on specific byte sequences being present based on the
         # parameters chosen above.
@@ -272,8 +300,17 @@ class TestPKCS7Builder(object):
         # as a separate section before the PKCS7 data. So we should expect to
         # have data in sig but not in sig_binary
         assert data in sig
+        # Parse the message to get the signed data, which is the
+        # first payload in the message
+        message = email.parser.BytesParser().parsebytes(sig)
+        signed_data = message.get_payload()[0].get_payload().encode()
         _pkcs7_verify(
-            serialization.Encoding.SMIME, sig, data, [cert], options, backend
+            serialization.Encoding.SMIME,
+            sig,
+            signed_data,
+            [cert],
+            options,
+            backend,
         )
         assert data not in sig_binary
         _pkcs7_verify(
@@ -285,7 +322,7 @@ class TestPKCS7Builder(object):
             backend,
         )
 
-    def test_sign_byteslike(self):
+    def test_sign_byteslike(self, backend):
         data = bytearray(b"hello world")
         cert, key = _load_cert_key()
         options = [pkcs7.PKCS7Options.DetachedSignature]
@@ -297,11 +334,36 @@ class TestPKCS7Builder(object):
 
         sig = builder.sign(serialization.Encoding.SMIME, options)
         assert bytes(data) in sig
+        _pkcs7_verify(
+            serialization.Encoding.SMIME,
+            sig,
+            data,
+            [cert],
+            options,
+            backend,
+        )
+
+        data = bytearray(b"")
+        builder = (
+            pkcs7.PKCS7SignatureBuilder()
+            .set_data(data)
+            .add_signer(cert, key, hashes.SHA256())
+        )
+
+        sig = builder.sign(serialization.Encoding.SMIME, options)
+        _pkcs7_verify(
+            serialization.Encoding.SMIME,
+            sig,
+            data,
+            [cert],
+            options,
+            backend,
+        )
 
     def test_sign_pem(self, backend):
         data = b"hello world"
         cert, key = _load_cert_key()
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         builder = (
             pkcs7.PKCS7SignatureBuilder()
             .set_data(data)
@@ -321,7 +383,6 @@ class TestPKCS7Builder(object):
     @pytest.mark.parametrize(
         ("hash_alg", "expected_value"),
         [
-            (hashes.SHA1(), b"\x06\x05+\x0e\x03\x02\x1a"),
             (hashes.SHA256(), b"\x06\t`\x86H\x01e\x03\x04\x02\x01"),
             (hashes.SHA384(), b"\x06\t`\x86H\x01e\x03\x04\x02\x02"),
             (hashes.SHA512(), b"\x06\t`\x86H\x01e\x03\x04\x02\x03"),
@@ -337,7 +398,7 @@ class TestPKCS7Builder(object):
             .set_data(data)
             .add_signer(cert, key, hash_alg)
         )
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         sig = builder.sign(serialization.Encoding.DER, options)
         assert expected_value in sig
         _pkcs7_verify(
@@ -347,13 +408,14 @@ class TestPKCS7Builder(object):
     @pytest.mark.parametrize(
         ("hash_alg", "expected_value"),
         [
-            (hashes.SHA1(), b"sha1"),
             (hashes.SHA256(), b"sha-256"),
             (hashes.SHA384(), b"sha-384"),
             (hashes.SHA512(), b"sha-512"),
         ],
     )
-    def test_sign_alternate_digests_detached(self, hash_alg, expected_value):
+    def test_sign_alternate_digests_detached(
+        self, hash_alg, expected_value, backend
+    ):
         data = b"hello world"
         cert, key = _load_cert_key()
         builder = (
@@ -370,7 +432,7 @@ class TestPKCS7Builder(object):
     def test_sign_attached(self, backend):
         data = b"hello world"
         cert, key = _load_cert_key()
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         builder = (
             pkcs7.PKCS7SignatureBuilder()
             .set_data(data)
@@ -398,7 +460,7 @@ class TestPKCS7Builder(object):
             .set_data(data)
             .add_signer(cert, key, hashes.SHA256())
         )
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         sig_no_binary = builder.sign(serialization.Encoding.DER, options)
         sig_binary = builder.sign(
             serialization.Encoding.DER, [pkcs7.PKCS7Options.Binary]
@@ -434,7 +496,7 @@ class TestPKCS7Builder(object):
             .add_signer(cert, key, hashes.SHA256())
         )
 
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         sig_binary = builder.sign(serialization.Encoding.DER, options)
         # LF gets converted to CR+LF (SMIME canonical form)
         # so data should not be present in the sig
@@ -466,10 +528,14 @@ class TestPKCS7Builder(object):
         # The text option adds text/plain headers to the S/MIME message
         # These headers are only relevant in SMIME mode, not binary, which is
         # just the PKCS7 structure itself.
-        assert b"text/plain" in sig_pem
-        # When passing the Text option the header is prepended so the actual
-        # signed data is this.
-        signed_data = b"Content-Type: text/plain\r\n\r\nhello world"
+        assert sig_pem.count(b"text/plain") == 1
+        assert b"Content-Type: text/plain\r\n\r\nhello world\r\n" in sig_pem
+        # Parse the message to get the signed data, which is the
+        # first payload in the message
+        message = email.parser.BytesParser().parsebytes(sig_pem)
+        signed_data = message.get_payload()[0].as_bytes(
+            policy=message.policy.clone(linesep="\r\n")
+        )
         _pkcs7_verify(
             serialization.Encoding.SMIME,
             sig_pem,
@@ -544,7 +610,7 @@ class TestPKCS7Builder(object):
             .add_signer(cert, key, hashes.SHA256())
         )
 
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         sig = builder.sign(serialization.Encoding.DER, options)
         assert sig.count(cert.public_bytes(serialization.Encoding.DER)) == 1
 
@@ -558,10 +624,11 @@ class TestPKCS7Builder(object):
         rsa_key = load_vectors_from_file(
             os.path.join("x509", "custom", "ca", "rsa_key.pem"),
             lambda pemfile: serialization.load_pem_private_key(
-                pemfile.read(), None
+                pemfile.read(), None, unsafe_skip_rsa_key_validation=True
             ),
             mode="rb",
         )
+        assert isinstance(rsa_key, rsa.RSAPrivateKey)
         rsa_cert = load_vectors_from_file(
             os.path.join("x509", "custom", "ca", "rsa_ca.pem"),
             loader=lambda pemfile: x509.load_pem_x509_certificate(
@@ -575,7 +642,7 @@ class TestPKCS7Builder(object):
             .add_signer(cert, key, hashes.SHA512())
             .add_signer(rsa_cert, rsa_key, hashes.SHA512())
         )
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         sig = builder.sign(serialization.Encoding.DER, options)
         # There should be three SHA512 OIDs in this structure
         assert sig.count(b"\x06\t`\x86H\x01e\x03\x04\x02\x03") == 3
@@ -594,7 +661,7 @@ class TestPKCS7Builder(object):
         rsa_key = load_vectors_from_file(
             os.path.join("x509", "custom", "ca", "rsa_key.pem"),
             lambda pemfile: serialization.load_pem_private_key(
-                pemfile.read(), None
+                pemfile.read(), None, unsafe_skip_rsa_key_validation=True
             ),
             mode="rb",
         )
@@ -605,13 +672,14 @@ class TestPKCS7Builder(object):
             ),
             mode="rb",
         )
+        assert isinstance(rsa_key, rsa.RSAPrivateKey)
         builder = (
             pkcs7.PKCS7SignatureBuilder()
             .set_data(data)
             .add_signer(cert, key, hashes.SHA384())
             .add_signer(rsa_cert, rsa_key, hashes.SHA512())
         )
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         sig = builder.sign(serialization.Encoding.DER, options)
         # There should be two SHA384 and two SHA512 OIDs in this structure
         assert sig.count(b"\x06\t`\x86H\x01e\x03\x04\x02\x02") == 2
@@ -627,7 +695,9 @@ class TestPKCS7Builder(object):
 
     def test_add_additional_cert_not_a_cert(self, backend):
         with pytest.raises(TypeError):
-            pkcs7.PKCS7SignatureBuilder().add_certificate(b"notacert")
+            pkcs7.PKCS7SignatureBuilder().add_certificate(
+                b"notacert"  # type: ignore[arg-type]
+            )
 
     def test_add_additional_cert(self, backend):
         data = b"hello world"
@@ -645,7 +715,7 @@ class TestPKCS7Builder(object):
             .add_signer(cert, key, hashes.SHA384())
             .add_certificate(rsa_cert)
         )
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         sig = builder.sign(serialization.Encoding.DER, options)
         assert (
             sig.count(rsa_cert.public_bytes(serialization.Encoding.DER)) == 1
@@ -668,8 +738,83 @@ class TestPKCS7Builder(object):
             .add_certificate(rsa_cert)
             .add_certificate(rsa_cert)
         )
-        options = []
+        options: typing.List[pkcs7.PKCS7Options] = []
         sig = builder.sign(serialization.Encoding.DER, options)
         assert (
             sig.count(rsa_cert.public_bytes(serialization.Encoding.DER)) == 2
         )
+
+
+@pytest.mark.supported(
+    only_if=lambda backend: backend.pkcs7_supported(),
+    skip_message="Requires OpenSSL with PKCS7 support",
+)
+class TestPKCS7SerializeCerts:
+    @pytest.mark.parametrize(
+        ("encoding", "loader"),
+        [
+            (serialization.Encoding.PEM, pkcs7.load_pem_pkcs7_certificates),
+            (serialization.Encoding.DER, pkcs7.load_der_pkcs7_certificates),
+        ],
+    )
+    def test_roundtrip(self, encoding, loader, backend):
+        certs = load_vectors_from_file(
+            os.path.join("pkcs7", "amazon-roots.der"),
+            lambda derfile: pkcs7.load_der_pkcs7_certificates(derfile.read()),
+            mode="rb",
+        )
+        p7 = pkcs7.serialize_certificates(certs, encoding)
+        certs2 = loader(p7)
+        assert certs == certs2
+
+    def test_ordering(self, backend):
+        certs = load_vectors_from_file(
+            os.path.join("pkcs7", "amazon-roots.der"),
+            lambda derfile: pkcs7.load_der_pkcs7_certificates(derfile.read()),
+            mode="rb",
+        )
+        p7 = pkcs7.serialize_certificates(
+            list(reversed(certs)), serialization.Encoding.DER
+        )
+        certs2 = pkcs7.load_der_pkcs7_certificates(p7)
+        assert certs == certs2
+
+    def test_pem_matches_vector(self, backend):
+        p7_pem = load_vectors_from_file(
+            os.path.join("pkcs7", "isrg.pem"),
+            lambda p: p.read(),
+            mode="rb",
+        )
+        certs = pkcs7.load_pem_pkcs7_certificates(p7_pem)
+        p7 = pkcs7.serialize_certificates(certs, serialization.Encoding.PEM)
+        assert p7 == p7_pem
+
+    def test_der_matches_vector(self, backend):
+        p7_der = load_vectors_from_file(
+            os.path.join("pkcs7", "amazon-roots.der"),
+            lambda p: p.read(),
+            mode="rb",
+        )
+        certs = pkcs7.load_der_pkcs7_certificates(p7_der)
+        p7 = pkcs7.serialize_certificates(certs, serialization.Encoding.DER)
+        assert p7 == p7_der
+
+    def test_invalid_types(self):
+        certs = load_vectors_from_file(
+            os.path.join("pkcs7", "amazon-roots.der"),
+            lambda derfile: pkcs7.load_der_pkcs7_certificates(derfile.read()),
+            mode="rb",
+        )
+        with pytest.raises(TypeError):
+            pkcs7.serialize_certificates(
+                "not a list of certs",  # type: ignore[arg-type]
+                serialization.Encoding.PEM,
+            )
+
+        with pytest.raises(TypeError):
+            pkcs7.serialize_certificates([], serialization.Encoding.PEM)
+
+        with pytest.raises(TypeError):
+            pkcs7.serialize_certificates(
+                certs, "not an encoding"  # type: ignore[arg-type]
+            )
