@@ -26,12 +26,11 @@ from cryptography.hazmat.primitives.ciphers import (
 from cryptography.hazmat.primitives.ciphers.modes import GCM
 from cryptography.hazmat.primitives.kdf.hkdf import HKDF, HKDFExpand
 from cryptography.hazmat.primitives.kdf.kbkdf import (
-    CounterLocation,
     KBKDFCMAC,
     KBKDFHMAC,
+    CounterLocation,
     Mode,
 )
-from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
 
 from ...utils import load_vectors_from_file
 
@@ -48,10 +47,8 @@ def _load_all_params(path, file_names, param_loader):
 def generate_encrypt_test(
     param_loader, path, file_names, cipher_factory, mode_factory
 ):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_encryption(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 encrypt_test(backend, cipher_factory, mode_factory, params)
 
@@ -81,13 +78,12 @@ def encrypt_test(backend, cipher_factory, mode_factory, params):
 def generate_aead_test(
     param_loader, path, file_names, cipher_factory, mode_factory
 ):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     assert mode_factory is GCM
-    # We don't support IVs < 64-bit in GCM mode so just strip them out
-    all_params = [i for i in all_params if len(i["iv"]) >= 16]
 
     def test_aead(self, backend, subtests):
+        all_params = _load_all_params(path, file_names, param_loader)
+        # We don't support IVs < 64-bit in GCM mode so just strip them out
+        all_params = [i for i in all_params if len(i["iv"]) >= 16]
         for params in all_params:
             with subtests.test():
                 aead_test(backend, cipher_factory, mode_factory, params)
@@ -106,23 +102,26 @@ def aead_test(backend, cipher_factory, mode_factory, params):
         # hex encoded.
         pytest.skip("Non-96-bit IVs unsupported in FIPS mode.")
 
+    tag = binascii.unhexlify(params["tag"])
+    mode = mode_factory(
+        binascii.unhexlify(params["iv"]),
+        tag,
+        len(tag),
+    )
+    assert isinstance(mode, GCM)
     if params.get("pt") is not None:
-        plaintext = params["pt"]
-    ciphertext = params["ct"]
-    aad = params["aad"]
+        plaintext = binascii.unhexlify(params["pt"])
+    ciphertext = binascii.unhexlify(params["ct"])
+    aad = binascii.unhexlify(params["aad"])
     if params.get("fail") is True:
         cipher = Cipher(
             cipher_factory(binascii.unhexlify(params["key"])),
-            mode_factory(
-                binascii.unhexlify(params["iv"]),
-                binascii.unhexlify(params["tag"]),
-                len(binascii.unhexlify(params["tag"])),
-            ),
+            mode,
             backend,
         )
         decryptor = cipher.decryptor()
-        decryptor.authenticate_additional_data(binascii.unhexlify(aad))
-        actual_plaintext = decryptor.update(binascii.unhexlify(ciphertext))
+        decryptor.authenticate_additional_data(aad)
+        actual_plaintext = decryptor.update(ciphertext)
         with pytest.raises(InvalidTag):
             decryptor.finalize()
     else:
@@ -132,34 +131,31 @@ def aead_test(backend, cipher_factory, mode_factory, params):
             backend,
         )
         encryptor = cipher.encryptor()
-        encryptor.authenticate_additional_data(binascii.unhexlify(aad))
-        actual_ciphertext = encryptor.update(binascii.unhexlify(plaintext))
+        encryptor.authenticate_additional_data(aad)
+        actual_ciphertext = encryptor.update(plaintext)
         actual_ciphertext += encryptor.finalize()
-        tag_len = len(binascii.unhexlify(params["tag"]))
-        assert binascii.hexlify(encryptor.tag[:tag_len]) == params["tag"]
+        assert encryptor.tag[: len(tag)] == tag
         cipher = Cipher(
             cipher_factory(binascii.unhexlify(params["key"])),
             mode_factory(
                 binascii.unhexlify(params["iv"]),
-                binascii.unhexlify(params["tag"]),
-                min_tag_length=tag_len,
+                tag,
+                min_tag_length=len(tag),
             ),
             backend,
         )
         decryptor = cipher.decryptor()
-        decryptor.authenticate_additional_data(binascii.unhexlify(aad))
-        actual_plaintext = decryptor.update(binascii.unhexlify(ciphertext))
+        decryptor.authenticate_additional_data(aad)
+        actual_plaintext = decryptor.update(ciphertext)
         actual_plaintext += decryptor.finalize()
-        assert actual_plaintext == binascii.unhexlify(plaintext)
+        assert actual_plaintext == plaintext
 
 
 def generate_stream_encryption_test(
     param_loader, path, file_names, cipher_factory
 ):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_stream_encryption(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 stream_encryption_test(backend, cipher_factory, params)
 
@@ -185,10 +181,8 @@ def stream_encryption_test(backend, cipher_factory, params):
 
 
 def generate_hash_test(param_loader, path, file_names, hash_cls):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_hash(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 hash_test(backend, hash_cls, params)
 
@@ -240,10 +234,8 @@ def base_hmac_test(backend, algorithm):
 
 
 def generate_hmac_test(param_loader, path, file_names, algorithm):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_hmac(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 hmac_test(backend, algorithm, params)
 
@@ -257,32 +249,6 @@ def hmac_test(backend, algorithm, params):
     assert h.finalize() == binascii.unhexlify(md.encode("ascii"))
 
 
-def generate_pbkdf2_test(param_loader, path, file_names, algorithm):
-    all_params = _load_all_params(path, file_names, param_loader)
-
-    def test_pbkdf2(self, backend, subtests):
-        for params in all_params:
-            with subtests.test():
-                pbkdf2_test(backend, algorithm, params)
-
-    return test_pbkdf2
-
-
-def pbkdf2_test(backend, algorithm, params):
-    # Password and salt can contain \0, which should be loaded as a null char.
-    # The NIST loader loads them as literal strings so we replace with the
-    # proper value.
-    kdf = PBKDF2HMAC(
-        algorithm,
-        int(params["length"]),
-        params["salt"],
-        int(params["iterations"]),
-        backend,
-    )
-    derived_key = kdf.derive(params["password"])
-    assert binascii.hexlify(derived_key) == params["derived_key"]
-
-
 def generate_aead_exception_test(cipher_factory, mode_factory):
     def test_aead_exception(self, backend):
         aead_exception_test(backend, cipher_factory, mode_factory)
@@ -291,9 +257,11 @@ def generate_aead_exception_test(cipher_factory, mode_factory):
 
 
 def aead_exception_test(backend, cipher_factory, mode_factory):
+    mode = mode_factory(binascii.unhexlify(b"0" * 24))
+    assert isinstance(mode, GCM)
     cipher = Cipher(
         cipher_factory(binascii.unhexlify(b"0" * 32)),
-        mode_factory(binascii.unhexlify(b"0" * 24)),
+        mode,
         backend,
     )
     encryptor = cipher.encryptor()
@@ -309,15 +277,18 @@ def aead_exception_test(backend, cipher_factory, mode_factory):
         encryptor.update(b"b" * 16)
     with pytest.raises(AlreadyFinalized):
         encryptor.finalize()
+
+    mode2 = mode_factory(binascii.unhexlify(b"0" * 24), b"0" * 16)
+    assert isinstance(mode2, GCM)
     cipher = Cipher(
         cipher_factory(binascii.unhexlify(b"0" * 32)),
-        mode_factory(binascii.unhexlify(b"0" * 24), b"0" * 16),
+        mode2,
         backend,
     )
     decryptor = cipher.decryptor()
     decryptor.update(b"a" * 16)
     with pytest.raises(AttributeError):
-        decryptor.tag
+        decryptor.tag  # type: ignore[attr-defined]
 
 
 def generate_aead_tag_exception_test(cipher_factory, mode_factory):
@@ -398,10 +369,8 @@ def hkdf_expand_test(backend, algorithm, params):
 
 
 def generate_hkdf_test(param_loader, path, file_names, algorithm):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_hkdf(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 hkdf_extract_test(backend, algorithm, params)
             with subtests.test():
@@ -413,17 +382,15 @@ def generate_hkdf_test(param_loader, path, file_names, algorithm):
 
 
 def generate_kbkdf_counter_mode_test(param_loader, path, file_names):
-    all_params = _load_all_params(path, file_names, param_loader)
-
     def test_kbkdf(self, backend, subtests):
-        for params in all_params:
+        for params in _load_all_params(path, file_names, param_loader):
             with subtests.test():
                 kbkdf_counter_mode_test(backend, params)
 
     return test_kbkdf
 
 
-def _kbkdf_hmac_counter_mode_test(backend, prf, ctr_loc, params):
+def _kbkdf_hmac_counter_mode_test(backend, prf, ctr_loc, brk_loc, params):
     supported_hash_algorithms: typing.Dict[
         str, typing.Type[hashes.HashAlgorithm]
     ] = {
@@ -449,13 +416,14 @@ def _kbkdf_hmac_counter_mode_test(backend, prf, ctr_loc, params):
         None,
         binascii.unhexlify(params["fixedinputdata"]),
         backend=backend,
+        break_location=brk_loc,
     )
 
     ko = ctrkdf.derive(binascii.unhexlify(params["ki"]))
     assert binascii.hexlify(ko) == params["ko"]
 
 
-def _kbkdf_cmac_counter_mode_test(backend, prf, ctr_loc, params):
+def _kbkdf_cmac_counter_mode_test(backend, prf, ctr_loc, brk_loc, params):
     supported_cipher_algorithms: typing.Dict[
         str, typing.Type[BlockCipherAlgorithm]
     ] = {
@@ -469,6 +437,10 @@ def _kbkdf_cmac_counter_mode_test(backend, prf, ctr_loc, params):
     algorithm = supported_cipher_algorithms.get(prf)
     assert algorithm is not None
 
+    # TripleDES is disallowed in FIPS mode.
+    if backend._fips_enabled and algorithm is algorithms.TripleDES:
+        pytest.skip("TripleDES is not supported in FIPS mode.")
+
     ctrkdf = KBKDFCMAC(
         algorithm,
         Mode.CounterMode,
@@ -480,6 +452,7 @@ def _kbkdf_cmac_counter_mode_test(backend, prf, ctr_loc, params):
         None,
         binascii.unhexlify(params["fixedinputdata"]),
         backend=backend,
+        break_location=brk_loc,
     )
 
     ko = ctrkdf.derive(binascii.unhexlify(params["ki"]))
@@ -490,37 +463,40 @@ def kbkdf_counter_mode_test(backend, params):
     supported_counter_locations = {
         "before_fixed": CounterLocation.BeforeFixed,
         "after_fixed": CounterLocation.AfterFixed,
+        "middle_fixed": CounterLocation.MiddleFixed,
     }
 
-    ctr_loc = supported_counter_locations.get(params.get("ctrlocation"))
-    if ctr_loc is None or not isinstance(ctr_loc, CounterLocation):
-        pytest.skip(
-            "Does not support counter location: {}".format(
-                params.get("ctrlocation")
-            )
-        )
-    del params["ctrlocation"]
+    ctr_loc = supported_counter_locations[params.pop("ctrlocation")]
+    brk_loc = None
+
+    if ctr_loc == CounterLocation.MiddleFixed:
+        assert "fixedinputdata" not in params
+        params["fixedinputdata"] = params.pop(
+            "databeforectrdata"
+        ) + params.pop("dataafterctrdata")
+
+        brk_loc = params.pop("databeforectrlen")
+        assert isinstance(brk_loc, int)
 
     prf = params.get("prf")
     assert prf is not None
     assert isinstance(prf, str)
     del params["prf"]
     if prf.startswith("hmac"):
-        _kbkdf_hmac_counter_mode_test(backend, prf, ctr_loc, params)
+        _kbkdf_hmac_counter_mode_test(backend, prf, ctr_loc, brk_loc, params)
     else:
         assert prf.startswith("cmac")
-        _kbkdf_cmac_counter_mode_test(backend, prf, ctr_loc, params)
+        _kbkdf_cmac_counter_mode_test(backend, prf, ctr_loc, brk_loc, params)
 
 
 def generate_rsa_verification_test(
     param_loader, path, file_names, hash_alg, pad_factory
 ):
-    all_params = _load_all_params(path, file_names, param_loader)
-    all_params = [
-        i for i in all_params if i["algorithm"] == hash_alg.name.upper()
-    ]
-
     def test_rsa_verification(self, backend, subtests):
+        all_params = _load_all_params(path, file_names, param_loader)
+        all_params = [
+            i for i in all_params if i["algorithm"] == hash_alg.name.upper()
+        ]
         for params in all_params:
             with subtests.test():
                 rsa_verification_test(backend, params, hash_alg, pad_factory)
