@@ -10,6 +10,7 @@ import os
 import pytest
 
 from cryptography import x509
+from cryptography.exceptions import UnsupportedAlgorithm
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec, ed448, ed25519, rsa
 from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15
@@ -27,17 +28,13 @@ def _load_data(filename, loader):
 
 
 def _cert_and_issuer():
-    from cryptography.hazmat.backends.openssl.backend import backend
-
     cert = _load_cert(
         os.path.join("x509", "cryptography.io.pem"),
         x509.load_pem_x509_certificate,
-        backend,
     )
     issuer = _load_cert(
         os.path.join("x509", "rapidssl_sha256_ca_g3.pem"),
         x509.load_pem_x509_certificate,
-        backend,
     )
     return cert, issuer
 
@@ -917,7 +914,7 @@ class TestOCSPResponseBuilder:
             None,
         )
 
-        with pytest.raises(ValueError):
+        with pytest.raises(UnsupportedAlgorithm):
             builder.sign(private_key, hashes.BLAKE2b(digest_size=64))
 
     def test_sign_none_hash_not_eddsa(self):
@@ -1045,12 +1042,9 @@ class TestOCSPResponse:
             os.path.join("x509", "ocsp", "resp-sha256.der"),
             ocsp.load_der_ocsp_response,
         )
-        from cryptography.hazmat.backends.openssl.backend import backend
-
         issuer = _load_cert(
             os.path.join("x509", "letsencryptx3.pem"),
             x509.load_pem_x509_certificate,
-            backend,
         )
         assert resp.response_status == ocsp.OCSPResponseStatus.SUCCESSFUL
         assert (
